import edu.neu.ccs.*;
import edu.neu.ccs.gui.*;
import edu.neu.ccs.util.*;
import edu.neu.ccs.filter.*;
import edu.neu.ccs.console.*;
import edu.neu.ccs.pedagogy.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;
import java.lang.reflect.*;

public class HanoiApplication extends DisplayPanel implements JPTConstants {
    
    /** The Hanoi model. */
    protected final HanoiModel model = new HanoiModel();
    
    /** The Hanoi view. */
    protected final HanoiView view = model.getHanoiView();
    
    
    /** The constructor. */
    public HanoiApplication() {
        makeGUI();
        
        model.randomHanoi(8);
    }
    
    protected Action[] makeMoveActions() {
        Action[] actions = new Action[6];
        int index = 0;
        
        for (int tower1 = 0; tower1 <= 2; tower1++)
            for (int tower2 = 0; tower2 <= 2; tower2++)
                if (tower1 != tower2) {
                    final int t1 = tower1;
                    final int t2 = tower2;
                    final String name = "Move disk: " + t1 + " to " + t2;
                    
                    actions[index] = new SimpleAction(name) {
                            public void perform() {
                                model.moveDisk(t1, t2);
                            }
                        };
                    
                    index++;
                }
        
        return actions;
    }

    protected void makeGUI() {
        ActionsPanel actionsPanel = new ActionsPanel(makeMoveActions());
        
        TablePanel panel = new TablePanel(
            new Object[] { view, actionsPanel }, VERTICAL, 5, 5, NORTH);
        
        add(panel);
    }
    
    public static void main(String[] args) {
        JPTFrame.createQuickJPTFrame("Hanoi", new HanoiApplication());
    }
}