import edu.neu.ccs.*;
import edu.neu.ccs.gui.*;
import edu.neu.ccs.util.*;
import edu.neu.ccs.filter.*;
import edu.neu.ccs.console.*;
import edu.neu.ccs.pedagogy.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;
import java.lang.reflect.*;

public class HanoiModel {
    
    /** Maximum number of disks. */
    public static int maxDisks = 8;
    
    /** Number of disks between 1 and maxDisks. */
    protected int disks = 1;
    
    /** Structure for the three Hanoi towers. */
    protected int[][] towers = new int[3][maxDisks];
    
    /** View for the three Hanoi towers. */
    protected HanoiView view = null;
    
    /** Constructor. */
    public HanoiModel() {
        view = new HanoiView(this);
    }
    
    /** Get the associated view. */
    public HanoiView getHanoiView() {
        return view;
    }
    
    /** Get function. */
    public int getSize(int tower, int level) {
        return towers[tower][level];
    }
    
    /** Set function. */
    public void setSize(int tower, int level, int value) {
        towers[tower][level] = value;
    }
    
    /**
     * Returns the top level in the tower with a value > 0
     * or -1 if no such value exists.
     */
    public int topLevel(int tower) {
        for (int level = maxDisks - 1; level >= 0; level--)
            if (getSize(tower, level) > 0)
                return level;
        
        return -1;
    }
    
    /** If possible, move the top level disk from tower1 to tower2. */
    public void moveDisk(int tower1, int tower2) {
        int level1 = topLevel(tower1);
        int level2 = topLevel(tower2) + 1;
        
        // return if tower1 has no disk or tower2 has no room
        if ((level1 == -1) || (level2 == maxDisks))
            return;
        
        int size = getSize(tower1, level1);
        
        setSize(tower1, level1, 0   );
        setSize(tower2, level2, size);
        
        view.refresh();
    }
    
    /** Clear the towers. */
    public void clear() {
        disks = 0;
        
        for (int tower = 0; tower <= 2; tower++)
            for (int level = 0; level < maxDisks; level++)
                setSize(tower, level, 0);
        
        view.refresh();
    }
    
    /** Initalize the given tower to have size disks. */
    public void initializeTower(int tower, int size) {
        clear();
        
        disks = size;
        
        int level = 0;
        
        while (size > 0) {
            setSize(tower, level, size);
            
            level++;
            size--;
        }
        
        view.refresh();
    }
    
    /** Create a random Hanoi configuration with size disks. */
    public void randomHanoi(int size) {
        clear();
        
        disks = size;
        
        while (size > 0) {
            int tower = MathUtilities.randomInt(0, 2);
            int level = topLevel(tower) + 1;
            
            setSize(tower, level, size);
            
            size--;
        }
        
        view.refresh();
    }
}